@@ -14,7 +14,7 @@ use Scalar::Does      qw/does/;
 use Carp;
 use namespace::clean;
 
-our $VERSION = '1.10';
+our $VERSION = '1.11';
 
 # builtin methods for "Limit-Offset" dialects
 my %limit_offset_dialects = (
@@ -429,15 +429,26 @@ sub bind_params {
   $sth->isa('DBI::st') or croak "sth argument is not a DBI statement handle";
   foreach my $i (0 .. $#bind) {
     my $val = $bind[$i];
-    if ((ref $val || '') eq 'SCALAR') {
+    my $ref = ref $val || '';
+    if ($ref eq 'SCALAR') {
+      # a scalarref is interpreted as an INOUT parameter
       $sth->bind_param_inout($i+1, $val, $INOUT_MAX_LEN);
     }
+    elsif ($ref eq 'ARRAY' && $self->looks_like_ternary_bind_param($val)) {
+      # call ternary form of DBI::bind_param
+      $sth->bind_param($i+1, @$val);
+    }
     else {
+      # other cases are passed directly to DBI::bind_param
       $sth->bind_param($i+1, $val);
     }
   }
 }
 
+sub looks_like_ternary_bind_param {
+  my ($self, $val) = @_;
+  return @$val == 2 && !ref($val->[0]) && does($val->[1], 'HASH');
+}
 
 
 #----------------------------------------------------------------------
@@ -585,6 +596,50 @@ sub _where_field_IN {
   }
 }
 
+#----------------------------------------------------------------------
+# override of parent's methods for decoding arrayrefs
+#----------------------------------------------------------------------
+
+sub _where_hashpair_ARRAYREF {
+  my ($self, $k, $v) = @_;
+
+  if ($self->looks_like_ternary_bind_param($v)) {
+    $self->_assert_no_bindtype_columns;
+    my $sql = CORE::join ' ', $self->_convert($self->_quote($k)),
+                              $self->_sqlcase($self->{cmp}),
+                              $self->_convert('?');
+    my @bind = ($v);
+    return ($sql, @bind);
+  }
+  else {
+    return $self->next::method($k, $v);
+  }
+}
+
+
+sub _where_field_op_ARRAYREF {
+  my ($self, $k, $op, $vals) = @_;
+
+  if ($self->looks_like_ternary_bind_param($vals)) {
+    $self->_assert_no_bindtype_columns;
+    my $sql = CORE::join ' ', $self->_convert($self->_quote($k)),
+                              $self->_sqlcase($op),
+                              $self->_convert('?');
+    my @bind = ($vals);
+    return ($sql, @bind);
+  }
+  else {
+    return $self->next::method($k, $op, $vals);
+  }
+}
+
+sub _assert_no_bindtype_columns {
+  my ($self) = @_;
+  $self->{bindtype} ne 'columns'
+    or croak 'values of shape [$val, \%type] are not compatible'
+           . 'with ...->new(bindtype => "columns")';
+}
+
 
 
 #----------------------------------------------------------------------
@@ -667,6 +722,14 @@ because it may possibly be useful for other needs.
                    ],
   );
 
+  ($sql, @bind) = $sqla->select(
+   -from     => 'Foo',
+   -where    => {bar => [$xml, {ora_type => ORA_XMLTYPE}]},
+  );
+  my $sth = $dbh->prepare($sql);
+  $sqla->bind_params($sth, @bind);
+  $sth->execute;
+
   my $merged = $sqla->merge_conditions($cond_A, $cond_B, ...);
   ($sql, @bind) = $sqla->select(..., -where => $merged, ..);
 
@@ -885,15 +948,24 @@ If omitted, C<< -columns >> takes '*' as default argument.
 =item C<< -from => $table || \@joined_tables >> 
 
 
-=item C<< -where => \%where >>
+=item C<< -where => $criteria >>
 
-C<< \%where >> is a reference to a hash or array of 
-criteria that will be translated into SQL clauses. In most cases, this
-will just be something like C<< {col1 => 'val1', col2 => 'val2'} >>;
-see L<SQL::Abstract::select|SQL::Abstract/select> for 
- detailed description of the
-structure of that hash or array. It can also be
-a plain SQL string like C<< "col1 IN (3, 5, 7, 11) OR col2 IS NOT NULL" >>.
+Like in L<SQL::Abstract>, C<< $criteria >> can be 
+a plain SQL string like C<< "col1 IN (3, 5, 7, 11) OR col2 IS NOT NULL" >>;
+but in most cases, it will rather be a reference to a hash or array of
+conditions that will be translated into SQL clauses, like
+for example C<< {col1 => 'val1', col2 => 'val2'} >>.
+The structure of that hash or array can be nested to express complex
+boolean combinations of criteria; see
+L<SQL::Abstract::select|SQL::Abstract/select> for a detailed description.
+
+Compared to C<SQL::Abstract>, criteria in C<SQL::Abstract::More> support
+one additional feature: values can be expressed as arrayrefs of shape
+C<< [$real_value, \%type] >>, suited for being passed to the ternary
+form of L<DBI/bind_param> : this is convenient when the DBD driver needs
+additional information about the values used in the statement. When using
+this features, also use L</bind_params> to perform the appropriate bindings
+before executing the statement.
 
 =item C<< -union => [ %select_subargs ] >>
 
@@ -1015,6 +1087,11 @@ parsing the C<-columns> parameter.
     -returning => $return_structure,
   );
 
+Like for L</select>, values assigned to columns can be expressed as
+arrayrefs of shape C<< [$real_value, \%type] >>, suited for being 
+passed to the ternary form of L<DBI/bind_param>. In that case,
+also use the L</bind_params> method.
+
 Named parameters to the C<insert()> method are just syntactic sugar
 for better readability of the client's code. Parameters
 C<-into> and C<-values> are passed verbatim to the parent method.
@@ -1064,9 +1141,10 @@ present module is there for help. Example:
     -where => \%conditions,
   );
 
+This works in the same spirit as the L</insert> method above.
 Named parameters to the C<update()> method are just syntactic sugar
 for better readability of the client's code; they are passed verbatim
-to the parent method. 
+to the parent method.
 
 
 =head2 delete
@@ -1228,21 +1306,45 @@ produces
 
   $sqla->bind_params($sth, @bind);
 
-For each value in C<@bind>, calls either
-L<DBI/bind_param_inout> (if the value is a scalarref),
-or L<DBI/bind_param> (otherwise). 
+For each C<$value> in C<@bind>:
 
-This method was meant especially as a convenience for Oracle 
-statements of shape "INSERT ... RETURNING ... INTO ..."
-(see L</insert> method above).
+=over 
 
-When calling L<DBI/bind_param_inout>, the value for C<$max_len>
-has been set arbitrarily to 99, which should be good enough 
-for most uses. Should you need another value, you can change it by
-setting 
+=item *
+
+if the value is a scalarref, call
+
+  $sth->bind_param_inout($index, $value, $INOUT_MAX_LEN)
+
+(see L<DBI/bind_param_inout>). C<$INOUT_MAX_LEN> defaults to
+99, which should be good enough for most uses; should you need another value, 
+you can change it by setting
 
   local $SQL::Abstract::More::INOUT_MAX_LEN = $other_value;
 
+=item *
+
+if the value is an arrayref of shape C<< [$real_value, \%attrs] >>,
+then call
+
+  $sth->bind_param($index, $real_value, \%attrs);
+
+where C<\%attrs> is usually a datatype specification for the DBD driver
+(see L<DBI/bind_param>)
+
+=item *
+
+for all other cases, call
+
+  $sth->bind_param($index, $value);
+
+=back
+
+This method is useful either as a convenience for Oracle
+statements of shape C<"INSERT ... RETURNING ... INTO ...">
+(see L</insert> method above), or as a way to indicate specific
+datatypes to the database driver.
+
 
 =head1 TODO
 
